/////////////////////////////////////////////////////////////////////////////
/*
  Copyright 2008 Ronald S. Burkey <info@sandroid.org>

  This file is part of GutenMark.

  GutenMark is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  GutenMark is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with GutenMark; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Filename:	main.cpp
  Purpose:	This is a simple GUI front-end for GutenMark.
  Mods:		04/18/2008 RSB	Began.
  
  This program is written using wxWidgets, and is intended to be as
  cross-platform as GutenMark is ... or at least as much as possible.
  Basically, all it does is to provide a way to do all of the same
  stuff as GutenMark, but to avoid the command-line.
  
  The user interface was designed with wxGlade, and the files
  GUItenMark.h and GUItenMark.cpp consist entirely of code generated
  by wxGlade.
  
  The event handlers in this file were all originally auto-generated
  to reside in GUItenMark.cpp.  For some reason that now escapes me,
  I thought it was better to move them into this separate file ...
  I guess, to make sure that wxGlade didn't overwrite them later.
  But it wouldn't have anyhow.
*/

#include "GUItenMark.h"
#include <wx/filename.h>


////////////////////////////////////////////////////////////////////////////
// Auxiliary functions not in MyFrame or MyApp classes.

//-----------------------------------------------------
// Define working/configuration directory.

#ifdef WIN32
#define DIRECTORY_DELIMITER "\\"
#else
#define DIRECTORY_DELIMITER "/"
#endif

class BaseDirectory {
public:
  BaseDirectory (void);
  wxString Name;
};

BaseDirectory::BaseDirectory (void)
{
#ifdef WIN32
  Name += "c:\\Program Files\\GutenMark";
#else
  Name = "/home/";
  Name += wxGetenv ("USER");
  Name += "/.GutenMark";
#endif
  wxMkdir (Name);
}

BaseDirectory *BaseDir = new BaseDirectory ();


////////////////////////////////////////////////////////////////////////////
// Functions for the MyApp class.

class MyApp: public wxApp
{
  public:
    virtual bool OnInit();
};

IMPLEMENT_APP (MyApp)

bool MyApp::OnInit()
{
  MyFrame *frame = new MyFrame( NULL, -1, wxT("") );
  frame->SideEffectsOfLists ();
  frame->GetConfiguration ("Default");
  frame->Maximize(TRUE);
  frame->Show(TRUE);
  SetTopWindow(frame);
  return TRUE;
} 


////////////////////////////////////////////////////////////////////////////
// Event handlers for the MyFrame class.

void MyFrame::OnClose (wxCloseEvent & event)
{
  PutConfiguration ("Default");
  Destroy ();
}

void MyFrame::ClickOutputFolderBrowse(wxCommandEvent &event)
{
    event.Skip();
    wxString Dir = wxDirSelector ("Choose a folder for output", OutputFolder->GetValue ());
    if (!Dir.empty ())
      OutputFolder->SetValue (Dir);
}

void MyFrame::ClickAddFiles(wxCommandEvent &event)
{
    //int i;
    //wxString Directory;
    wxArrayString Filenames;
    event.Skip();
    //i = InputFiles->GetCount ();
    //if (!i)
    //  Directory = "";
    //else
    //  Directory = wxPathOnly (InputFiles->GetString (i - 1));
    wxFileDialog *Dialog = new wxFileDialog (this, "Choose etext file(s)",
    					     Directory, "", 
					     "Text files (*.txt)|*.txt|"
					     "All files (*.*)|*.*",
					     wxFD_DEFAULT_STYLE | 
					     wxFD_FILE_MUST_EXIST | 
					     wxFD_MULTIPLE |
					     wxFD_CHANGE_DIR);  
    if (wxID_OK == Dialog->ShowModal ())
      {
	Dialog->GetPaths (Filenames);  
	InputFiles->Append (Filenames);
	Directory = Dialog->GetDirectory ();
	SideEffectsOfLists ();
      }
    delete Dialog;
}

void MyFrame::ClickClearInputFiles(wxCommandEvent &event)
{
  event.Skip();
  InputFiles->Clear ();
  SideEffectsOfLists ();
}

void MyFrame::ClickConvertTexts(wxCommandEvent &event)
{
  wxArrayString Filenames;
  int Count, i, j, Skip = 0;
  
  event.Skip();
  Filenames = InputFiles->GetStrings ();
  for (i = 0, Count = Filenames.GetCount (); i < Count; i++)
    {
      //---------------------------------------------------------------
      // Construct the GutenMark command line.
      
      // Program name.
      wxString CommandLine = wxT ("\"") + BaseDir->Name + 
      		    wxT (DIRECTORY_DELIMITER "binary") +
		    wxT (DIRECTORY_DELIMITER "GutenMark\"");
		    
      // --author
      if (AuthorOverride->IsEnabled () && !AuthorOverride->IsEmpty ())
        {
          CommandLine += " \"--author=";
	  CommandLine += AuthorOverride->GetValue ();
	  CommandLine += "\"";
	}
	
      // --caps-ok
      if (AllCapsOk->GetValue ())
        CommandLine += " --caps-ok";

      if (!ConfigurationFile->IsEmpty ())
        {
          CommandLine += " \"--config=";
	  CommandLine += ConfigurationFile->GetValue ();
	  CommandLine += "\"";
	}
	
      // --first-capital
      if (FirstWordAllCapsOk->GetValue ())
        CommandLine += " --first-capital";

      // --first-italics
      if (FirstWordItalicsOk->GetValue ())
        CommandLine += " --first-italics";

      // --force-symbolic/--force-numeric
      if (HtmlOutput->GetValue () && HtmlUseSymbolicEntities->GetValue ())
        CommandLine += " --force-symbolic";
      else
        CommandLine += " --force-numeric";

      // --latex
      if (LatexOutput->GetValue ())
        CommandLine += " --latex";

      // --latex
      if (LatexOutput->GetValue () && LatexSectionMarkup->GetValue ())
        CommandLine += " --latex-sections";

      // --no-diacritical
      if (!RestoreDiacrits->GetValue ())
        CommandLine += " --no-diacritical";

      // --no-foreign
      if (!ItalicizeNonEnglish->GetValue ())
        CommandLine += " --no-foreign";

      // --no-justify
      if (RaggedRightMargin->GetValue ())
        CommandLine += " --no-justify";

      // --no-mdash
      if (PreserveStringsOfHyphens->GetValue ())
        CommandLine += " --no-mdash";

      // --no-parskip
      if (LatexOutput->GetValue () && LatexIndentParagraphs->GetValue ())
        CommandLine += " --no-parskip";

      // --no-prefatory
      if (HtmlOutput->GetValue () && HtmlAddPrefatoryMarks->GetValue ())
        CommandLine += " --no-prefatory";

      // --no-toc
      if (LatexOutput->GetValue () && LatexEliminateToc->GetValue ())
        CommandLine += " --no-toc";

      // --page-breaks
      if (HtmlOutput->GetValue () && HtmlAddPageBreakMarks->GetValue ())
        CommandLine += " --page-breaks";

      // --profile
      j = Profile->GetSelection ();
      if (wxNOT_FOUND != j)
        {
	  CommandLine += " \"--profile=";
	  CommandLine += Profile->GetString (j);
	  CommandLine += "\"";
	}

      // --single-space
      if (SingleSpaceBetweenSentences->GetValue ())
        CommandLine += " --single-space";

      // --title
      if (TitleOverride->IsEnabled () && !TitleOverride->IsEmpty ())
        {
          CommandLine += " \"--title=";
	  CommandLine += TitleOverride->GetValue ();
	  CommandLine += "\"";
	}
	
      // --yes-header
      if (RetainProjectGutenbergHeader->GetValue ())
        CommandLine += " --yes-header";
	
      // --debug
      if (Debug->GetValue ())
        CommandLine += " --debug";

      // --ron
      if (RonsOptions->GetValue ())
        CommandLine += " --ron";

      // Input file.
      CommandLine += " \"";
      CommandLine += Filenames[i];
      CommandLine += "\"";
      
      // Output file.
      wxString OutputFile;
      wxFileName *Input = new wxFileName (Filenames[i]);
      OutputFile = OutputFolder->GetValue () + 
      		   wxT (DIRECTORY_DELIMITER) +
		   Input->GetName ();
      if (LatexOutput->GetValue ())
        OutputFile += ".tex";
      else
        OutputFile += ".html";
      CommandLine += " \"";
      CommandLine += OutputFile;
      CommandLine += "\"";

      //---------------------------------------------------------------
      // Now convert the text.
      wxBeginBusyCursor ();
      printf ("Command line:  %s\n", CommandLine.c_str ());
      long ConversionCode = wxExecute (CommandLine, wxEXEC_SYNC);
      wxEndBusyCursor ();
      
      //---------------------------------------------------------------
      // Now remove the converted file from the input list, and put it
      // in the output list.
      if (Debug->GetValue ())
        {
	  wxString LogFile;
	  LogFile = "Log file: ";
	  LogFile += Input->GetPath ();
	  LogFile += wxT (DIRECTORY_DELIMITER "GutenMark.log");
	  OutputFiles->Append (LogFile);
	}
      if (ConversionCode)
        {
	  OutputFile += wxT (" (Conversion error ");
	  OutputFile += wxString::Format ("%ld", ConversionCode);
	  OutputFile += wxT (")");
	  OutputFiles->Append (OutputFile);
	  Skip++;
	}
      else
        {
          InputFiles->Delete (Skip);
          OutputFiles->Append (OutputFile);
        }
	
      delete Input;
      
    }
    
  // All done!
  SideEffectsOfLists ();
}

void MyFrame::ClickClearOutputList(wxCommandEvent &event)
{
  event.Skip();
  OutputFiles->Clear ();
  SideEffectsOfLists ();
}

void MyFrame::ClickHtmlOutput(wxCommandEvent &event)
{
  event.Skip();
  if (HtmlOutput->GetValue ())
    {
      LatexSectionMarkup->Disable ();
      LatexIndentParagraphs->Disable ();
      LatexEliminateToc->Disable ();
      HtmlUseSymbolicEntities->Enable ();
      HtmlAddPrefatoryMarks->Enable ();
      HtmlAddPageBreakMarks->Enable ();
      RonsOptions->SetValue (FALSE);
    }
}

void MyFrame::ClickLatexOutput(wxCommandEvent &event){
  event.Skip();
  if (LatexOutput->GetValue ())
    {
      LatexSectionMarkup->Enable ();
      LatexIndentParagraphs->Enable ();
      LatexEliminateToc->Enable ();
      HtmlUseSymbolicEntities->Disable ();
      HtmlAddPrefatoryMarks->Disable ();
      HtmlAddPageBreakMarks->Disable ();
    }
}

void MyFrame::ClickRon (wxCommandEvent &event) {
  event.Skip ();
  LatexOutput->SetValue (TRUE);
  LatexIndentParagraphs->SetValue (TRUE);
  ClickLatexOutput (event);
}

void MyFrame::ClickLatexIndentParagraphs (wxCommandEvent &event) {
  event.Skip ();
  if (!LatexIndentParagraphs->GetValue ())
    RonsOptions->SetValue (FALSE);
  ClickLatexOutput (event);
}

void MyFrame::NewConfigurationFiles(wxCommandEvent &event){
#define MAXSTRING 1024
  FILE *fp;
  char *s, *ss;
  event.Skip();
  
  // What we need to do here is to parse the configuration
  // file to see what profiles are available in it.  The
  // profiles (in regex terms) match "^\[.*\]$".
  Profile->Clear ();
  fp = fopen (ConfigurationFile->GetValue (), "r");
  if (fp != NULL)
    {
      s = (char *) malloc (MAXSTRING);
      while (NULL != fgets (s, MAXSTRING, fp))
        if (*s == '[')
	  {
	    for (ss = s + 1; *ss && *ss != ']'; ss++);
	    if (*ss == ']')
	      {
	        *ss = 0;
		Profile->Append (s + 1);
	      }
	  }
      fclose (fp);
      free (s);
    }
  Profile->SetSelection (0);
#undef MAXSTRING
}

void MyFrame::ClickConfigurationFileBrowse(wxCommandEvent &event){
    event.Skip();
    wxString Path = wxPathOnly ( ConfigurationFile->GetValue ());
    wxString File = wxFileSelector ("Choose a GutenMark configuration file", 
    				    Path, "", "",
				    "Configuration files (*.cfg)|*.cfg|"
				    "All files (*.*)|*.*");
    if (!File.empty ())
      {
        ConfigurationFile->SetValue (File);
	NewConfigurationFiles (event);
      }
}

void MyFrame::ClickProfile(wxCommandEvent &event){
  event.Skip();
}

void MyFrame::ClickSaveSettings(wxCommandEvent &event){
    event.Skip();
    wxString Path = BaseDir->Name + wxT (DIRECTORY_DELIMITER "GUIConfigs");
    wxString File = wxFileSelector ("Save your GUItenMark configuration as", 
    				    Path, "", "", "*", wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
    if (!File.empty ())
      {
        wxFileName *Filename = new wxFileName (File);
        PutConfiguration (Filename->GetName ());
	delete Filename;
      }
}

void MyFrame::ClickRestoreSettings(wxCommandEvent &event){
    event.Skip();
    wxString Path = BaseDir->Name + wxT (DIRECTORY_DELIMITER "GUIConfigs");
    wxString File = wxFileSelector ("Restore your GUItenMark configuration from", 
    				    Path, "", "", "*", wxFD_OPEN | wxFD_FILE_MUST_EXIST);
    if (!File.empty ())
      {
        wxFileName *Filename = new wxFileName (File);
        GetConfiguration (Filename->GetName ());
	delete Filename;
      }
}

void MyFrame::ClickGoodbye(wxCommandEvent &event){
  event.Skip();
  Close ();
}

void MyFrame::ClickDefaultSettings(wxCommandEvent &event){
  event.Skip();
  GetConfiguration ("ThisFilenameDoesNotExistSoTheHardCodedDefaultIsUsed");
}

////////////////////////////////////////////////////////////////////////////
// Non-event-handler auxiliary functions which I've added to the MyFrame class.

void MyFrame::SideEffectsOfLists (void)
{
  switch (InputFiles->GetCount ())
    {
    case 0:
      ClearInputList->Disable ();
      ConvertTexts->Disable ();
      AuthorOverride->Disable ();
      AuthorLabel->Disable ();
      TitleOverride->Disable ();
      TitleLabel->Disable ();
      break;
    case 1:
      ClearInputList->Enable ();
      ConvertTexts->Enable ();
      AuthorOverride->Enable ();
      AuthorLabel->Enable ();
      TitleOverride->Enable ();
      TitleLabel->Enable ();
      break;
    default:
      ClearInputList->Enable ();
      ConvertTexts->Enable ();
      AuthorOverride->Enable ();
      AuthorLabel->Enable ();
      TitleOverride->Disable ();
      TitleLabel->Disable ();
      break;
    }
  if (OutputFiles->GetCount ())
    ClearOutputList->Enable ();
  else
    ClearOutputList->Disable ();
}

// Get a saved configuration identified by Name.  (Name is not a filename,
// but a base from which a filename is constructed by prefixing a path and
// suffixing an extension.)  Returns 0 on success, non-zero on failure.  
// "Failure" means that that configuration doesn't exist.  In this case, the
// configuration is replaced by a fallback configuration of my choosing.
// On entry to the program, there is an automatic call with Name=="Default".
int MyFrame::GetConfiguration (wxString Name)
{
  FILE *fp;
  int ReturnValue = 1, i;
  wxCommandEvent DummyEvent;
  wxString CfgName;
  char *s;
  
  s = (char *) malloc (2048);
  if (s == NULL)
    return (ReturnValue);
  
  // Configure according to my preferences.
  OutputFolder->SetValue (BaseDir->Name + wxT (DIRECTORY_DELIMITER "Outputs"));
  AllCapsOk->SetValue (FALSE);
  RestoreDiacrits->SetValue (TRUE);
  RaggedRightMargin->SetValue (FALSE);
  HtmlOutput->SetValue (TRUE);
  ClickHtmlOutput (DummyEvent);
  FirstWordAllCapsOk->SetValue (FALSE);
  ItalicizeNonEnglish->SetValue (TRUE);
  PreserveStringsOfHyphens->SetValue (FALSE);
  HtmlUseSymbolicEntities->SetValue (FALSE);
  LatexSectionMarkup->SetValue (FALSE);
  FirstWordItalicsOk->SetValue (FALSE);
  SingleSpaceBetweenSentences->SetValue (FALSE);
  HtmlAddPrefatoryMarks->SetValue (FALSE);
  LatexIndentParagraphs->SetValue (TRUE);
  RetainProjectGutenbergHeader->SetValue (FALSE);
  HtmlAddPageBreakMarks->SetValue (FALSE);
  LatexEliminateToc->SetValue (FALSE);
  ConfigurationFile->SetValue (BaseDir->Name + 
  			       wxT (DIRECTORY_DELIMITER "GutConfigs") +
  			       wxT (DIRECTORY_DELIMITER "GutenMark.cfg"));
  
  NewConfigurationFiles (DummyEvent);
  // Note that NewConfigurationFiles automatically sets Profile.  However,
  // I don't necessarily like the default selection it makes.
  i = Profile->FindString ("english");
  if (i != wxNOT_FOUND)
    Profile->SetSelection (i);
  Directory = "";
  Debug->SetValue (FALSE);
  RonsOptions->SetValue (FALSE);
  
  // Fetch the named configuration.
  CfgName = BaseDir->Name + 
  	    wxT (DIRECTORY_DELIMITER "GUIConfigs" DIRECTORY_DELIMITER) + Name;
  fp = fopen (CfgName, "r");
  if (fp != NULL)
    {
      int i;
      
      if (1 == fscanf (fp, "OutputFolder=%[^\n]\n", s))
        OutputFolder->SetValue (s);
      else
        goto Done;
      if (1 == fscanf (fp, "AllCapsOk=%d\n", &i))
        AllCapsOk->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "RestoreDiacrits=%d\n", &i))
        RestoreDiacrits->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "RaggedRightMargin=%d\n", &i))
        RaggedRightMargin->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "HtmlOutput=%d\n", &i))
        {
	  if (i)
	    {
	      HtmlOutput->SetValue (TRUE);
	      ClickHtmlOutput (DummyEvent);
	    }
	  else
	    {
	      LatexOutput->SetValue (TRUE);
	      ClickLatexOutput (DummyEvent);
	    }
	}
      else
        goto Done;
      if (1 == fscanf (fp, "FirstWordAllCapsOk=%d\n", &i))
        FirstWordAllCapsOk->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "ItalicizeNonEnglish=%d\n", &i))
        ItalicizeNonEnglish->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "PreserveStringsOfHyphens=%d\n", &i))
        PreserveStringsOfHyphens->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "HtmlUseSymbolicEntities=%d\n", &i))
        HtmlUseSymbolicEntities->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "LatexSectionMarkup=%d\n", &i))
        LatexSectionMarkup->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "FirstWordItalicsOk=%d\n", &i))
        FirstWordItalicsOk->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "SingleSpaceBetweenSentences=%d\n", &i))
        SingleSpaceBetweenSentences->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "HtmlAddPrefatoryMarks=%d\n", &i))
        HtmlAddPrefatoryMarks->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "LatexIndentParagraphs=%d\n", &i))
        LatexIndentParagraphs->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "RetainProjectGutenbergHeader=%d\n", &i))
        RetainProjectGutenbergHeader->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "HtmlAddPageBreakMarks=%d\n", &i))
        HtmlAddPageBreakMarks->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "LatexEliminateToc=%d\n", &i))
        LatexEliminateToc->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "ConfigurationFile=%[^\n]\n", s))
        ConfigurationFile->SetValue (s);
      else
        goto Done;
      NewConfigurationFiles (DummyEvent);
      if (1 == fscanf (fp, "Profile=%d\n", &i))
        Profile->SetSelection (i);
      else
        goto Done;
      if (1 == fscanf (fp, "InputDirectory=%[^\n]\n", s))
        Directory = s;
      else
        goto Done;
      if (1 == fscanf (fp, "Debug=%d\n", &i))
        Debug->SetValue (i);
      else
        goto Done;
      if (1 == fscanf (fp, "RonsOptions=%d\n", &i))
        RonsOptions->SetValue (i);
      else
        goto Done;
      fclose (fp);
    }
   
  ReturnValue = 0;
Done:
  free (s);
  return (ReturnValue);
}

// Write a saved configuration identified by Name.  (Name is not a filename,
// but a base from which a filename is constructed by prefixing a path and
// suffixing an extension.)  Returns 0 on success, non-zero on failure.  
// On exit from the program, there is an automatic call with Name=="Default".
int MyFrame::PutConfiguration (wxString Name)
{
  FILE *fp;
  int ReturnValue = 0;
  wxString CfgName;

  CfgName = BaseDir->Name + 
  	    wxT (DIRECTORY_DELIMITER "GUIConfigs" DIRECTORY_DELIMITER) + Name;
  fp = fopen (CfgName, "w");
  if (fp != NULL)
    {
      fprintf (fp, "OutputFolder=%s\n", OutputFolder->GetValue ().c_str ());
      fprintf (fp, "AllCapsOk=%d\n", AllCapsOk->GetValue ());
      fprintf (fp, "RestoreDiacrits=%d\n", RestoreDiacrits->GetValue ());
      fprintf (fp, "RaggedRightMargin=%d\n", RaggedRightMargin->GetValue ());
      fprintf (fp, "HtmlOutput=%d\n", HtmlOutput->GetValue ());
      fprintf (fp, "FirstWordAllCapsOk=%d\n", FirstWordAllCapsOk->GetValue ());
      fprintf (fp, "ItalicizeNonEnglish=%d\n", ItalicizeNonEnglish->GetValue ());
      fprintf (fp, "PreserveStringsOfHyphens=%d\n", PreserveStringsOfHyphens->GetValue ());
      fprintf (fp, "HtmlUseSymbolicEntities=%d\n", HtmlUseSymbolicEntities->GetValue ());
      fprintf (fp, "LatexSectionMarkup=%d\n", LatexSectionMarkup->GetValue ());
      fprintf (fp, "FirstWordItalicsOk=%d\n", FirstWordItalicsOk->GetValue ());
      fprintf (fp, "SingleSpaceBetweenSentences=%d\n", SingleSpaceBetweenSentences->GetValue ());
      fprintf (fp, "HtmlAddPrefatoryMarks=%d\n", HtmlAddPrefatoryMarks->GetValue ());
      fprintf (fp, "LatexIndentParagraphs=%d\n", LatexIndentParagraphs->GetValue ());
      fprintf (fp, "RetainProjectGutenbergHeader=%d\n", RetainProjectGutenbergHeader->GetValue ());
      fprintf (fp, "HtmlAddPageBreakMarks=%d\n", HtmlAddPageBreakMarks->GetValue ());
      fprintf (fp, "LatexEliminateToc=%d\n", LatexEliminateToc->GetValue ());
      fprintf (fp, "ConfigurationFile=%s\n", ConfigurationFile->GetValue ().c_str ());
      fprintf (fp, "Profile=%d\n", Profile->GetSelection ());
      fprintf (fp, "InputDirectory=%s\n", Directory.c_str ());
      fprintf (fp, "Debug=%d\n", Debug->GetValue ());
      fprintf (fp, "RonsOptions=%d\n", RonsOptions->GetValue ());
      fclose (fp);
    }
  
  return (ReturnValue);
}

